# This script demonstrates how to load single-trial AC data and plot the raster response for a single neuron to a single stimulus
# from Pennington & David 2023

import numpy as np
import os
import matplotlib.pyplot as plt

from nems0 import get_setting
from nems0.recording import load_recording
from nems0 import xforms, preprocessing, modelspec, epoch


# CONFIGURATION

file_list=[
'A1_TNC015a_abf3608afd9b5c83bf8ed17a5ac61040e9c83261.tgz',
'A1_TNC020a_2f0624a284401ad0e40680435fc57de3873e0a2c.tgz',
'A1_ARM030a_9b0586bd962b60ab7b02a21771c98695134b8be8.tgz',
'A1_TNC008a_2eb34ed73bb45f087a72652b5fcea810a9f2e9c9.tgz',
'A1_ARM029a_aefada7c479857e55fdafe3e00e94df189250651.tgz',
'A1_TNC021a_89668bbfb0d89473b8efff14b7b6625bb58060dd.tgz',
'A1_TNC017a_2d5a6c489bf573e2f762eb2e17ca5478271fcc64.tgz',
'A1_TNC012a_c677f8485dfd121765cd31de50230196a8a4bcc1.tgz',
'A1_TNC010a_2f50864a77d8ec2f1786ebd8b0161114a9e905b1.tgz',
'A1_TNC014a_a0ebe478cdce9612658c29d6766859fd66f1a3b6.tgz',
'A1_TNC016a_13decf694dbf004e611c8e362e23b6fa7852ee4b.tgz',
'A1_DRX006b_adb77e6b989c6b08bf7fce0c19ab1a94ba124399.tgz',
'A1_DRX007a_5409c3cb81c2745cec8af5f9a1402ed24ea7ec7d.tgz',
'A1_DRX008b_240d4f51148b270f6dc099eaccd4f316ce3021f5.tgz',
'A1_ARM032a_d28371c918efb9917c560f41e51ff15efdf516c5.tgz',
'A1_CRD017c_07010f7f2781fc649e4f1f90679212e60a7ff3b4.tgz',
'A1_CRD016d_f9cf97eab58415d6187e5f104e9df6bf06a9fd99.tgz',
'A1_TNC013a_cc40dccc6e141410c8b0c16e403b763ad368b170.tgz',
'A1_ARM031a_e73a3420ba4e26d680d9a8adc5bef1c32f6d9617.tgz',
'A1_ARM033a_8bc7cdda34517574d7973a1b6352d52d873bad7b.tgz',
'A1_TNC009a_91819235d1188908cee2787e5769d3613fbd756f.tgz',
'A1_TNC018a_2d5a31aeb27af29f52739c37061da96fcb058e96.tgz']

#  LOAD THE DATA
signals_dir = "file://"
modelspecs_dir = "file://"
cellid = "DRX006b-128-2"
filebase = file_list[11]

datafile = os.path.join(signals_dir, 'A1_single_sites', filebase)
rec = load_recording(datafile)

resp=rec['resp'].rasterize()
single_cell_resp = resp.extract_channels([cellid])

# find all stimuli in the validation set, identified by event names that start with "STIM_00"
val_epochs = epoch.epoch_names_matching(resp.epochs,"^STIM_00")
print("Validation epochs:", val_epochs)

# extract responses to a single validation stimulus
print("Plotting data for epoch", val_epochs[0], "cell", cellid)
r = single_cell_resp.extract_epoch(val_epochs[0])


# plot the raster
raster = r[:,0,:]
y,_,x=np.where(r)
x_sec=x/1000 - 0.25
plt.figure()
plt.scatter(x_sec,y,s=1, color='black')
plt.title(f"Raster, cell: {cellid} stim: {val_epochs[0]}")
plt.xlabel('time from stimulus onset (s)')
plt.ylabel('trial')


# plot a heatmap of the average population response
pop_raster = resp.extract_epoch(val_epochs[0])
pop_psth=pop_raster.mean(axis=0)

plt.figure()
plt.imshow(pop_psth, aspect='auto', cmap='gray_r', interpolation='none',origin='lower', extent=[-0.25, 1.25, 1, pop_psth.shape[0]])
plt.title(f"Population response, site: {cellid.split('-')[0]} stim: {val_epochs[0]}")
plt.xlabel('time from stimulus onset (s)')
plt.ylabel('unit')


